/*
   IMAGE.C
   Convert 1.44HD Mac-Formatted diskettes to .images and vice-versa

   The vMac utilities project.

   Turbo C/C++ [DOS 16bit]
   (c) Yoav Shadmi, 1997-1998
*/

#include <malloc.h>
#include <bios.h>
#include <conio.h>
#include <fcntl.h>
#include <io.h>
#include <stdio.h>
#include <string.h>
#include <sys\stat.h>
#include <process.h>

struct diskinfo_t dinfo;


void format (int verbose, int drive)
{
   struct diskinfo_t disk;
   char *dbuf;
   int  j, i, result;

   disk.drive    =  drive;
   disk.sector   =   1;
   disk.nsectors =  18;

   dbuf = malloc(9216);

   for (i=0 ; i<160 ; i++) {
      for (j=0; j<9216 ; j++)
	 dbuf[j] = 0x00;

      disk.buffer = dbuf;

      disk.head  = i % 2;
      disk.track = i / 2;

      result = (_bios_disk(3, &disk) & 0xff00) >> 8;
      if (result != 0) {
	 printf("Disk error\n", result);
	 exit(1);
      } else
	 if (verbose)
	    printf ("formatting %c: - cylinder %d : head %d\r", drive+65, i / 2, i % 2);
   }
}

int image_to_disk (int verbose, int drive, const char *filename)
{
  int result, handle, i;
  unsigned bytes;
  char dbuf[9216];

  format(verbose, drive);
  printf("\n");

  if ((handle = open(filename, O_RDONLY | O_BINARY)) == -1) {
     printf("%s not found.\n", filename);
     return 1;
  }
  for (i=0 ; i<160 ; i++)  {
    if ((bytes =_read(handle, dbuf, 9216)) != 9216) {
      printf("%s corrupt.\n", filename);
      return 1;
    }
    else {
      if (bytes==9216)
	 result = biosdisk(3, drive, (i % 2), (i / 2), 1, 18, dbuf);
	if (result != 0) {
	   printf("disk error.\n");
	   return 1;
	}
	if (verbose)
	   printf ("writing %c: - cylinder %d : head %d\r", drive+65, i / 2, i % 2);
    }
  }
  return 0;
}


int disk_to_image (int verbose, int drive, const char *filename)
{
   int handle, result;
   static char dbuf[9216];

   dinfo.drive 		=  drive;		// drive number
   dinfo.head  		=  0;    		// disk head number
   dinfo.track		=  0;    		// track number
   dinfo.sector  	=  1;  			// sector number
   dinfo.nsectors 	=  18; 			// sector count
   dinfo.buffer 	=  dbuf; 		// data buffer

   if (verbose) printf("Attempting to read from drive A:\n");

   result = _bios_disk(_DISK_READ, &dinfo);
   result = _bios_disk(_DISK_READ, &dinfo);

   if ((result & 0xff00) == 0) {
      if (verbose) printf("Disk read from A: successful.\n");
      _fmode = O_BINARY;   handle = creat(filename, S_IWRITE);

      for (dinfo.track=0 ; dinfo.track<=79 ; dinfo.track++) {
	 for (dinfo.head=0 ; dinfo.head<=1 ; dinfo.head++) {
	    if ((_bios_disk(_DISK_READ, &dinfo) & 0xff00) != 0) {
	       printf("disk error\n");
	       return 1;
	    }
	    if (_write(handle, dbuf, 9216) != 9216) {
	       printf("file error\n");
	       return 1;
	    }
	    if (verbose) {
	       printf ("reading %c: - cylinder %d : head %d\r", drive+65, dinfo.track, dinfo.head);
	    }
	 }
      }
      close(handle);
   }
   else {
      printf("Cannot read drive %c:", drive+65);
      return 1;
   }
   return 0;
}

void show_help(void)
{
   printf("usage: image [command] [mode] drive: file     \n\n");
   printf("command:                                        \n");
   printf("  r       : read disk into image file           \n");
   printf("  w       : write image file to disk          \n\n");
   printf("mode:  				           \n");
   printf("  -v      : verbose mode                        \n");
   printf("  -q      : quiet mode                          \n");
}

void main(int argc, char **argv)
{
   int drive;
   int verbose;

   printf("image.exe\n");
   if (argc>4) {
      if (strcmp(strlwr(argv[1]), "r") == 0) {
	 if (strcmp(strlwr(argv[2]), "-v") == 0)
	    verbose=1;
	 else if (strcmp(strlwr(argv[2]), "-q") == 0)
		 verbose=0;
	      else show_help();
	 if (strcmp(strlwr(argv[3]), "a:") == 0)
	    drive=0;
	 else if (strcmp(strlwr(argv[3]), "b:") == 0)
		  drive=1;
	      else show_help();
	 (void) disk_to_image(verbose, drive, argv[4]);
      }
      else if (strcmp(strlwr(argv[1]), "w") == 0) {
	      if (strcmp(strlwr(argv[2]), "-v") == 0)
		 verbose=1;
	      else if (strcmp(strlwr(argv[2]), "-q") == 0)
		      verbose=0;
		   else show_help();
	      if (strcmp(strlwr(argv[3]), "a:") == 0)
		 drive=0;
	      else if (strcmp(strlwr(argv[3]), "b:") == 0)
		      drive=1;
		   else show_help();
	      (void) image_to_disk(verbose, drive, argv[4]);
	   } else
	       show_help();

   } else
      show_help();
}
